<?php
/**
 * Plugin Name:  BuddyPress User Badges
 * Version:      1.2.7
 * Plugin URI:   https://buddydev.com/plugins/buddypress-user-badges
 * Description:  This plugin allows site admin to create and assign badges to his/her site users.
 * Author:       BuddyDev
 * Author URI:   https://buddydev.com/
 * Requires PHP: 5.3
 * License:      GPL2
 * License URI:  https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:  buddypress-user-badges
 * Domain Path:  /languages
 *
 * @package buddypress-user-badges
 **/

/**
 * Contributor: Ravi Sharma(raviousprime)
 */

use BuddyDev\BuddyPress_User_Badges\Bootstrap\Autoloader;
use BuddyDev\BuddyPress_User_Badges\Bootstrap\Bootstrapper;
use BuddyDev\BuddyPress_User_Badges\Schema\Schema;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Class BuddyPress_User_Badges
 *
 * @property-read $path     string Absolute path to the plugin directory.
 * @property-read $url      string Absolute url to the plugin directory.
 * @property-read $basename string Plugin base name.
 * @property-read $version  string Plugin version.
 */
class BuddyPress_User_Badges {

	/**
	 * Plugin Version.
	 *
	 * @var string
	 */
	private $version = '1.2.7';

	/**
	 * Class instance
	 *
	 * @var BuddyPress_User_Badges
	 */
	private static $instance = null;

	/**
	 * Plugin absolute directory path
	 *
	 * @var string
	 */
	private $path;

	/**
	 * Plugin absolute directory url
	 *
	 * @var string
	 */
	private $url;

	/**
	 * Plugin Basename.
	 *
	 * @var string
	 */
	private $basename;

	/**
	 * User information about badges
	 *
	 * @var array
	 */
	public $user_badges = array();

	/**
	 * Errors thrown by plugin
	 *
	 * @var WP_Error|string
	 */
	public $error = '';

	/**
	 * Protected properties. These properties are inaccessible via magic method.
	 *
	 * @var array
	 */
	private $guarded = array( 'instance' );

	/**
	 * BuddyPress_User_Badges constructor.
	 */
	private function __construct() {
		$this->bootstrap();
	}

	/**
	 * Get Singleton Instance
	 *
	 * @return BuddyPress_User_Badges
	 */
	public static function get_instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Bootstrap the core.
	 */
	private function bootstrap() {
		$this->path     = plugin_dir_path( __FILE__ );
		$this->url      = plugin_dir_url( __FILE__ );
		$this->basename = plugin_basename( __FILE__ );
		$this->error    = new WP_Error();

		// Load autoloader.
		require_once $this->path . 'src/bootstrap/class-autoloader.php';

		$autoloader = new Autoloader( 'BuddyDev\\BuddyPress_User_Badges\\', __DIR__ . '/src/' );

		spl_autoload_register( $autoloader );

		register_activation_hook( __FILE__, array( $this, 'on_activation' ) );
		register_deactivation_hook( __FILE__, array( $this, 'on_deactivation' ) );

		// Drop tables on uninstall.
		// register_uninstall_hook( __FILE__, array( 'Schema', 'drop' ) );.

		Bootstrapper::boot();
	}

	/**
	 * On activation create table
	 */
	public function on_activation() {
		Schema::create();

		if ( ! get_option( 'bp_user_badges_settings' ) ) {
			require_once $this->path . 'src/core/bp-user-badges-functions.php';
			$settings = bp_user_badges_get_default_options();
			update_option( 'bp_user_badges_settings', $settings );

			if ( is_multisite() ) {
				update_site_option( 'bp_user_badges_settings', $settings );
			}
		}
	}

	/**
	 * On deactivation. Do cleanup if needed.
	 */
	public function on_deactivation() {
		// do cleanup.
	}

	/**
	 * Check if the plugin is network active.
	 *
	 * @return bool
	 */
	public function is_network_active() {
		static $is_network_active;

		if ( ! is_null( $is_network_active ) ) {
			return $is_network_active;
		}

		if ( ! is_multisite() ) {
			$is_network_active = false;

			return false;
		}

		// Check the sitewide plugins array.
		$base    = $this->basename;
		$plugins = get_site_option( 'active_sitewide_plugins' );

		if ( ! is_array( $plugins ) || ! isset( $plugins[ $base ] ) ) {
			$is_network_active = false;
		} else {
			$is_network_active = true;
		}

		return $is_network_active;
	}

	/**
	 * Magic method for accessing property as readonly(It's a lie, references can be updated).
	 *
	 * @param string $name property name.
	 *
	 * @return mixed|null
	 */
	public function __get( $name ) {

		if ( ! in_array( $name, $this->guarded, true ) && property_exists( $this, $name ) ) {
			return $this->{$name};
		}

		return null;
	}
}

/**
 * Helper to access singleton instance
 *
 * @return BuddyPress_User_Badges
 */
function bp_user_badges() {
	return BuddyPress_User_Badges::get_instance();
}

bp_user_badges();
