<?php
/**
 * Admin Helper class.
 *
 * @package    buddypress-user-budges
 * @subpackage Admin
 * @copyright  Copyright (c) 2018, BuddyDev.Com
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyDev\BuddyPress_User_Badges\Admin;

// Exit if file accessed directly over web.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Admin_Helper
 */
class Admin_Helper {

	/**
	 * Admin Menu slug
	 *
	 * @var string
	 */
	private $menu_slug;

	/**
	 * Boot settings
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup settings
	 */
	public function setup() {
		$this->menu_slug = 'badges';

		add_action( 'admin_init', array( $this, 'init' ) );
		add_action( 'admin_menu', array( $this, 'add_menu' ) );
		add_action( 'admin_notices', array( $this, 'show_errors' ) );
	}

	/**
	 * Initialize the admin settings panel and fields
	 */
	public function init() {
		$page_slug = isset( $_GET['page'] ) ? trim( $_GET['page'] ) : '';

		if ( $this->menu_slug != $page_slug ) {
			return;
		}

		if ( ! current_user_can( 'edit_users' ) ) {
			return;
		}

		if ( isset( $_POST['bp_user_badges_save_btn'] ) ) {
			$this->save_custom_badges_detail();
		} elseif ( isset( $_POST['bp_user_badges_roles_badges_save_btn'] ) ) {
			$this->save_role_badges_detail();
		} elseif ( isset( $_POST['bp_user_badges_member_type_badges_save_btn'] ) ) {
			$this->save_member_type_badges_detail();
		} elseif ( isset( $_GET['bp_user_badges_action'] ) && 'delete-badge' == $_GET['bp_user_badges_action'] ) {
			$this->delete_badge();
		}
	}

	/**
	 * Add Menu
	 */
	public function add_menu() {

		if ( ! is_super_admin() ) {
			return;
		}

		$hook = add_users_page(
			_x( 'Badges', 'Admin page title', 'buddypress-user-badges' ),
			_x( 'Badges', 'Admin menu label', 'buddypress-user-badges' ),
			'create_users',
			$this->menu_slug,
			array( $this, 'render_badges' )
		);

		add_action( "admin_print_styles-{$hook}", array( $this, 'enqueue_assets' ) );
	}

	/**
	 * Enqueue styles.
	 */
	public function enqueue_assets() {
		wp_enqueue_style( 'bp_user_badges_admin_css' );
	}

	/**
	 * Show errors
	 */
	public function show_errors() {
		if ( ! current_user_can( 'edit_users' ) ) {
			return;
		}
		$class = 'notice notice-error is-dismissible';

		$error = bp_user_badges()->error;

		foreach ( $error->get_error_codes() as $code ) {
			printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $error->get_error_message( $code ) );
		}
	}

	/**
	 * Render badges
	 */
	public function render_badges() {

		wp_enqueue_media();
		wp_enqueue_script( 'bp_user_badges_admin_js' );

		require_once bp_user_badges()->path . 'src/admin/templates/badges.php';
	}

	/**
	 * Save details
	 */
	private function save_custom_badges_detail() {
		$posted_data = wp_unslash( $_POST ); // WPCS: Input var okay.

		if ( ! wp_verify_nonce( $posted_data['_wpnonce'], 'bp-user-badges-save-badges' ) ) { // WPCS: Input var okay.
			wp_die( esc_html_e( 'Unable to save details', 'buddypress-user-badges' ) );
		}

		$this->save_details( $posted_data, 'custom', 'custom' );
	}

	/**
	 * Save details
	 */
	private function save_role_badges_detail() {
		$posted_data = wp_unslash( $_POST ); // WPCS: Input var okay.

		if ( ! wp_verify_nonce( $posted_data['_wpnonce'], 'bp-user-badges-save-role-badges' ) ) { // WPCS: Input var okay.
			wp_die( esc_html_e( 'Unable to save details', 'buddypress-user-badges' ) );
		}

		$this->save_details( $posted_data, 'predefined', 'role' );
	}

	/**
	 * Save details
	 */
	private function save_member_type_badges_detail() {
		$posted_data = wp_unslash( $_POST ); // WPCS: Input var okay.

		if ( ! wp_verify_nonce( $posted_data['_wpnonce'], 'bp-user-badges-save-member-type-badges' ) ) { // WPCS: Input var okay.
			wp_die( esc_html_e( 'Unable to save details', 'buddypress-user-badges' ) );
		}

		$this->save_details( $posted_data, 'predefined', 'member_type' );
	}

	/**
	 * Delete badge
	 *
	 * @return bool|int
	 */
	private function delete_badge() {
		$badge_id = $_GET['badge_id'] ? absint( $_GET['badge_id'] ) : 0;

		if ( empty( $badge_id ) ) {
			return false;
		}

		if ( ! wp_verify_nonce( $_GET['_wpnonce'], 'bp-user-badges-delete-badge' ) ) {
			wp_die( esc_html_e( 'Unable to delete badge', 'buddypress-user-badges' ) );
		}

		return bp_user_badges_delete_badge( $badge_id );
	}

	/**
	 * Save details
	 *
	 * @param array  $posted_data Posted data.
	 * @param string $source Source.
	 * @param string $context Context.
	 */
	private function save_details( $posted_data, $source = 'custom', $context = 'custom' ) {
		$count = count( $posted_data['bp_user_badges_name'] ); // WPCS: Input var okay.
		$error = bp_user_badges()->error;

		for ( $i = 0; $i < $count; $i ++ ) {
			$name = empty( $posted_data['bp_user_badges_name'][ $i ] ) ? '' : sanitize_text_field( $posted_data['bp_user_badges_name'][ $i ] );

			$slug        = empty( $posted_data['bp_user_badges_slug'][ $i ] ) ? sanitize_title_with_dashes( $posted_data['bp_user_badges_name'][ $i ] ) : sanitize_text_field( $posted_data['bp_user_badges_slug'][ $i ] );
			$description = empty( $posted_data['bp_user_badges_description'][ $i ] ) ? '' : sanitize_textarea_field( $posted_data['bp_user_badges_description'][ $i ] );
			$image       = empty( $posted_data['bp_user_badges_image'][ $i ] ) ? '' : esc_url( $posted_data['bp_user_badges_image'][ $i ] );
			$link        = empty( $posted_data['bp_user_badges_link'][ $i ] ) ? '' : sanitize_text_field( $posted_data['bp_user_badges_link'][ $i ] );
			$is_active   = empty( $posted_data['bp_user_badges_is_active'][ $i ] ) ? 0 : 1;
			$badge_id    = empty( $posted_data['bp_user_badges_badge_id'][ $i ] ) ? '' : sanitize_text_field( $posted_data['bp_user_badges_badge_id'][ $i ] );

			$args = array(
				'name'        => $name,
				'slug'        => $slug,
				'description' => $description,
				'image'       => $image,
				'link'        => $link,
				'position'    => $i,
				'source'      => $source,
				'context'     => $context,
				'is_active'   => $is_active,
			);

			// If name is empty will not be added to badges list.
			if ( $is_active && empty( $name ) ) {
				$error->add( 'empty_name', __( 'Name is mandatory field.', 'buddypress-user-badges' ) );
			}

			if ( $is_active && empty( $slug ) ) {
				$error->add( 'empty_slug', __( 'Slug can not be empty.', 'buddypress-user-badges' ) );
			}

			if ( $is_active && empty( $image ) ) {
				$error->add( 'empty_image', sprintf( 'Image can not be empty for %s', $args['name'] ) );
			}

			// Only check for custom badges. If existed or not.
			if ( 'custom' == $source ) {
				$exists = bp_user_badges_badge_exists(
					array(
						'slug'    => $args['slug'],
						'context' => $context,
					)
				);

				if ( $is_active && $exists && empty( $badge_id ) ) {
					$error->add( 'already_existed', sprintf( 'Badge with name %s already existed', $posted_data['bp_user_badges_name'][ $i ] ) );
				}
			}

			do_action( 'bp_user_badges_pre_validate', $args, $error );

			// Do not process if has errors.
			if ( $error->has_errors() ) {
				continue;
			}

			if ( $badge_id ) {
				bp_user_badges_update_badge( $args, array( 'id' => $badge_id ) );
				wp_cache_delete( 'badges_all', 'bp-user-badges' );
			} else {
				bp_user_badges_add_badge( $args );
			}
		}
	}
}
