<?php
/**
 * Admin Settings Pages Helper.
 *
 * @package    buddypress-user-budges
 * @subpackage Admin
 * @copyright  Copyright (c) 2018, BuddyDev.Com
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyDev\BuddyPress_User_Badges\Admin;

use \Press_Themes\PT_Settings\Page;

// Exit if file accessed directly over web.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Admin_Settings_Helper
 */
class Admin_Settings_Helper {
	/**
	 * Setting page slug
	 *
	 * @var string
	 */
	private $setting_page_slug;

	/**
	 * Used to keep a reference of the Page, It will be used in rendering the view.
	 *
	 * @var \Press_Themes\PT_Settings\Page
	 */
	private $page;

	/**
	 * Boot class
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup callbacks
	 */
	private function setup() {
		$this->setting_page_slug = 'bp-user-badges-settings';

		add_action( 'admin_menu', array( $this, 'add_menu' ) );
		add_action( 'admin_init', array( $this, 'init' ) );
	}

	/**
	 * Show/render the setting page
	 */
	public function render_settings() {
		$this->page->render();
	}

	/**
	 * Add setting menu
	 */
	public function add_menu() {

		if ( ! is_super_admin() ) {
			return;
		}

		add_options_page(
			_x( 'Badge settings', 'Admin settings page title', 'buddypress-user-badges' ),
			_x( 'Badge settings', 'Admin setting menu label', 'buddypress-user-badges' ),
			'create_users',
			$this->setting_page_slug,
			array( $this, 'render_settings' )
		);
	}

	/**
	 * Initialize the admin settings panel and fields
	 */
	public function init() {
		global $pagenow;
		if ( ! current_user_can( 'edit_users' ) ) {
			return;
		}

		$page_slug = isset( $_GET['page'] ) ? trim( $_GET['page'] ) : '';

		if ( 'options.php' === $pagenow || $this->setting_page_slug === $page_slug ) {
			$this->register_settings();
		}
	}

	/**
	 * Register settings
	 */
	private function register_settings() {
		$page = new Page( 'bp_user_badges_settings', __( 'Badges', 'buddypress-user-badges' ) );

		// General settings tab.
		$general         = $page->add_panel( 'general', _x( 'General', 'Admin settings panel title', 'buddypress-user-badges' ) );
		$section_general = $general->add_section( 'settings', '' );

		$defaults = bp_user_badges_get_default_options();

		$fields = array(
			array(
				'name'    => 'enabled_badge_types',
				'label'   => _x( 'Auto assign badges based on', 'Admin Settings', 'buddypress-user-badges' ),
				'type'    => 'multicheck',
				'options' => array(
					'role'        => __( 'Roles', 'buddypress-user-badges' ),
					'member_type' => __( 'Member types', 'buddypress-user-badges' ),
				),
				'default' => $defaults['enabled_badge_types'],
				'desc'    => __( 'In addition to custom badges, You can created badges for selected item types too. e.g. for each role site admin can add badge.', 'buddypress-user-badges' ),
			),
			array(
				'name'    => 'role_badges_position',
				'label'   => _x( 'Role badges position', 'Admin Settings', 'buddypress-user-badges' ),
				'type'    => 'select',
				'options' => array(
					'before_custom' => __( 'Before custom badges', 'buddypress-user-badges' ),
					'after_custom'  => __( 'After custom badges', 'buddypress-user-badges' ),
				),
				'default' => $defaults['role_badges_position'],
			),
			array(
				'name'    => 'member_type_badges_position',
				'label'   => _x( 'Member types badges position', 'Admin Settings', 'buddypress-user-badges' ),
				'type'    => 'select',
				'options' => array(
					'before_custom' => __( 'Before custom badges', 'buddypress-user-badges' ),
					'after_custom'  => __( 'After custom badges', 'buddypress-user-badges' ),
				),
				'default' => $defaults['member_type_badges_position'],
			),
			array(
				'name'    => 'badge_display_context',
				'label'   => _x( 'Where should we show user badges?', 'Admin Settings', 'buddypress-user-badges' ),
				'type'    => 'multicheck',
				'options' => bp_user_badges_get_display_context_options(),
				'default' => $defaults['badge_display_context'],
			),
			array(
				'name'    => 'badge_position',
				'label'   => _x( 'Badge Position', 'Admin settings', 'buddypress-user-badges' ),
				'type'    => 'select',
				'options' => array(
					'before_name' => __( 'Before name', 'buddypress-user-badges' ),
					'after_name'  => __( 'After name', 'buddypress-user-badges' ),
				),
				'default' => $defaults['badge_position'],
			),
			array(
				'name'    => 'disable_css',
				'label'   => _x( 'Disable css?', 'Admin settings', 'buddypress-user-badges' ),
				'type'    => 'select',
				'options' => array(
					0 => __( 'No', 'buddypress-user-badges' ),
					1 => __( 'Yes', 'buddypress-user-badges' ),
				),
				'default' => $defaults['disable_css'],
				'desc'    => __( ' The included css, sets width to 32px to limit badge dimension. You can disable it here and use your own css if you want.', 'bp-user-badges' ),
			),
		);

		$section_general->add_fields( $fields );

		do_action( 'bp_user_badges_admin_settings_page', $page );

		$this->page = $page;

		// allow enabling options.
		$page->init();
	}
}