<?php
/**
 * Core functions file
 *
 * @package buddypress-user-badges
 */

use BuddyDev\BuddyPress_User_Badges\Models\Badge;
use BuddyDev\BuddyPress_User_Badges\Models\Badge_Relationship;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Get setting
 *
 * @param string $option_name Setting name.
 *
 * @return mixed
 */
function bp_user_badges_get_option( $option_name = '' ) {

	if ( is_multisite() && bp_user_badges()->is_network_active() ) {
		$options = get_site_option( 'bp_user_badges_settings', bp_user_badges_get_default_options() );
	} else {
		$options = get_option( 'bp_user_badges_settings', bp_user_badges_get_default_options() );
	}

	if ( isset( $options[ $option_name ] ) ) {
		return $options[ $option_name ];
	}

	return null;
}

/**
 * Get default options.
 *
 * @return array
 */
function bp_user_badges_get_default_options() {
	return array(
		'enabled_badge_types'         => array(
			'role'        => 'role',
			'member_type' => 'member_type',
		),
		'role_badges_position'        => 'before_custom',
		'member_type_badges_position' => 'before_custom',
		'badge_display_context'       => array(
			'members_directory'  => 'members_directory',
			'activity'           => 'activity',
			'group_members_list' => 'group_members_list',
			'user_profile'       => 'user_profile',
		),
		'badge_position'              => 'after_name',
		'disable_css'                 => 0,
	);
}

/**
 * Add new badge
 *
 * @param array $args Args.
 *
 * @return bool
 */
function bp_user_badges_add_badge( $args ) {

	$args = bp_parse_args(
		$args,
		array(
			'name'        => '',
			'slug'        => '',
			'description' => '',
			'image'       => '',
			'link'        => '',
			'position'    => '',
			'source'      => '',
			'context'     => '',
			'is_active'   => 0,
		),
		'add_badge_args'
	);

	$badge = Badge::create( $args );

	$badge_id = $badge->save();

	if ( $badge_id ) {
		do_action( 'bp_user_badges_badge_added', $badge_id, $badge );
		wp_cache_delete( 'badges_all', 'bp-user-badges' );

	}

	return $badge_id;
}

/**
 * Update badge
 *
 * @param array $args Array.
 * @param array $where_args Where conditions array.
 *
 * @return bool
 */
function bp_user_badges_update_badge( $args, $where_args ) {

	$args = bp_parse_args(
		$args,
		array(
			'name'        => '',
			'slug'        => '',
			'description' => '',
			'image'       => '',
			'link'        => '',
			'position'    => '',
			'source'      => '',
			'context'     => '',
			'is_active'   => 0,
		),
		'update_badge_args'
	);

	$update = Badge::update( $args, $where_args );

	if ( $update ) {
		do_action( 'bp_user_badges_badge_updated', $args, $where_args );
		wp_cache_delete( 'badges_all', 'bp-user-badges' );
	}

	return $update;
}

/**
 * Delete badge
 *
 * @param int $id Badge id.
 *
 * @return bool|int
 */
function bp_user_badges_delete_badge( $id ) {

	$delete = Badge::destroy( array( 'id' => $id ) );

	if ( ! $delete ) {
		return false;
	}

	do_action( 'bp_user_badges_badge_deleted', $id );
	wp_cache_delete( 'badges_all', 'bp-user-badges' );

	return $delete;
}

/**
 * Get badge details
 *
 * @param int $id Badge id.
 *
 * @return Badge|null
 */
function bp_user_badges_get_badge( $id ) {

	$badges = bp_user_badges_get_all_badges();
	return isset( $badges[ $id ] ) ? $badges[ $id ] : null;
}

/**
 * Get badge details
 *
 * @param string $slug Badge slug.
 * @param string $type Badge type.
 *
 * @return Badge|null
 */
function bp_user_badges_get_badge_by_slug( $slug, $type = '' ) {

	$filter = array();

	if ( $type ) {
		$filter['context'] = $type;
	}

	$badges = bp_user_badges_get_all_badges( $filter );

	foreach ( $badges as $badge ) {
		if ( $slug == $badge->slug ) {
			return $badge;
		}
	}

	return null;
}

/**
 * Get all badges.
 *
 * @param array $filter filter.
 *
 * @return array
 */
function bp_user_badges_get_all_badges( $filter = array() ) {

	$badges = wp_cache_get( 'badges_all', 'bp-user-badges' );

	if ( false === $badges ) {
		// we should replace it.
		$all_badges = Badge::all();
		$badges     = array();

		foreach ( $all_badges as $badge ) {
			$badges[ $badge->id ] = $badge;
		}

		wp_cache_set( 'badges_all', $badges, 'bp-user-badges' );
	}

	if ( empty( $badges ) || empty( $filter ) ) {
		return (array) $badges;
	}

	return wp_list_filter( $badges, $filter );
}

/**
 * Check if badge existed or not
 *
 * @param array $args Array.
 *
 * @return int
 */
function bp_user_badges_badge_exists( $args ) {

	$args = wp_parse_args(
		$args,
		array(
			'slug'    => '',
			'context' => '',
		)
	);

	return Badge::exists( $args );
}

/**
 * Get badges
 *
 * @param array $args Get query parameters.
 *
 * @todo rename in future.
 *
 * @return Badge_Relationship[]
 */
function bp_user_badges_get_relationships( $args = array() ) {
	return Badge_Relationship::get( $args );
}

/**
 * Get user badges ids
 *
 * @param int $user_id User id.
 *
 * @return Badge[]|null
 */
function bp_user_badges_get_user_badges( $user_id ) {
	return (array) apply_filters( 'bp_user_badges_user_badges', bp_user_badges_get_user_badges_by_type( $user_id, 'custom' ), $user_id );
}


/**
 * Get user badges ids
 *
 * @param int    $user_id User id.
 * @param string $type badge type('custom', 'role', 'member_type').
 * @return Badge[]|null
 */
function bp_user_badges_get_user_badges_by_type( $user_id, $type = '' ) {

	$bp_user_badges = bp_user_badges();

	$badges          = $bp_user_badges->user_badges;
	$user_badges_ids = isset( $badges[ $user_id ] ) ? $badges[ $user_id ] : null;

	if ( is_null( $user_badges_ids ) ) {
		$user_badges_relationships = bp_user_badges_get_relationships(
			array(
				'user_id'  => $user_id,
				'per_page' => 0,
			)
		);

		$user_badges_ids = array();
		foreach ( $user_badges_relationships as $user_badges_relationship ) {
			$user_badges_ids[] = $user_badges_relationship->badge_id;
		}
		$bp_user_badges->user_badges[ $user_id ] = $user_badges_ids;
	}

	if ( empty( $user_badges_ids ) ) {
		return array();
	}

	// populate objects array.
	$active_badges = array();

	foreach ( $user_badges_ids as $badge_id ) {
		$found_badge = bp_user_badges_get_badge( $badge_id );
		if ( $found_badge ) {
			$active_badges[] = $found_badge;
		}
	}
	usort( $active_badges, 'bp_user_badges_compare_badges_by_position' );

	if ( $type ) {
		return wp_list_filter( $active_badges, array( 'context' => $type ) );
	}

	return $active_badges;
}

/**
 * Add badge to user
 *
 * @param int $badge_id Badge id.
 * @param int $user_id User id.
 *
 * @return bool
 */
function bp_user_badges_add_badge_to_user( $badge_id, $user_id ) {

	$exists = Badge_Relationship::exists(
		array(
			'user_id'  => $user_id,
			'badge_id' => $badge_id,
		)
	);

	if ( $exists ) {
		return false;
	}

	$badge_relationship = Badge_Relationship::create(
		array(
			'user_id'  => $user_id,
			'badge_id' => $badge_id,
		)
	);

	$added = $badge_relationship->save();

	do_action( 'bp_user_badges_user_badge_added', $badge_id, $user_id );

	return $added;
}

/**
 * Remove user badges
 *
 * @param int $user_id User id.
 *
 * @return bool|int
 */
function bp_user_badges_remove_user_badges( $user_id ) {
	return Badge_Relationship::destroy( array( 'user_id' => $user_id ) );
}

/**
 * Remove relationship based on badge id.
 *
 * @param int $badge_id Badge id.
 *
 * @return bool|int
 */
function bp_user_badges_remove_badge_relationships( $badge_id ) {
	return Badge_Relationship::destroy( array( 'badge_id' => $badge_id ) );
}

/**
 * Sort badges
 *
 * @param Badge $a Badge object.
 * @param Badge $b Badge object.
 *
 * @return int
 */
function bp_user_badges_compare_badges_by_position( $a, $b ) {

	$a_pos = absint( $a->position );
	$b_pos = absint( $b->position );

	if ( $a_pos === $b_pos ) {
		return 0;
	}

	return ( $a_pos < $b_pos ) ? - 1 : 1;
}

/**
 * Get user badges
 *
 * @param int $user_id User id.
 *
 * @return string
 */
function bp_user_badges_get_user_badges_html( $user_id ) {

	$user_badges = bp_user_badges_get_user_badges( $user_id );

	if ( empty( $user_badges ) ) {
		return '';
	}

	$position = bp_user_badges_get_option( 'badge_position' );

	$position_class = ( 'after_name' === $position ) ? 'after-name' : 'before-name';

	$badges = '<span class="bp-user-badges-badge-list ' . $position_class . '">';
	foreach ( $user_badges as $user_badge ) {
		// Continue if badge not active.
		if ( ! $user_badge->is_active ) {
			continue;
		}

		$badges .= sprintf( '<a href="%s" title="%s" data-balloon="%s" data-balloon-pos="up"><img src="%s" alt="%s" /></a>', esc_url( $user_badge->link ), esc_attr( $user_badge->description ), esc_attr( $user_badge->description ), esc_url( $user_badge->image ), esc_attr( $user_badge->name ) );
	}
	$badges .= '</span>';

	return $badges;
}


/**
 * Locate or load a file from theme or fallback to plugin.
 *
 * @param string $template templates.
 * @param bool   $load load template?.
 *
 * @return string
 */
function bp_user_badges_locate_template( $template, $load = false ) {

	$_template = 'buddypress/buddypress-user-badges/' . $template;

	$located = locate_template( $_template, false, false );

	if ( ! $located ) {
		$path = bp_user_badges()->path . 'templates/buddypress-user-badges/';

		if ( is_readable( $path . $template ) ) {
			$located = $path . $template;
		}
	}

	if ( ! $load ) {
		return $located;
	}

	require $located;
}

/**
 * Get display context options for badges
 *
 * @return array
 */
function bp_user_badges_get_display_context_options() {
	$options = array(
		'members_directory' => __( 'Members Directory', 'buddypress-user-badges' ),
		'user_profile'      => __( 'User Profile', 'buddypress-user-badges' ),
	);

	/*if ( bp_is_active( 'activity' ) ) {
		$options['activity'] = __( 'Activity', 'buddypress-user-badges' );
	}*/

	if ( bp_is_active( 'groups' ) ) {
		$options['group_members_list'] = __( 'Group Members List', 'buddypress-user-badges' );
	}

	if ( function_exists( 'bbpress' ) ) {
		$options['bbp_single_topic'] = __( 'bbPress - Single Topic Screen', 'buddypress-user-badges' );
	}

	return $options;
}
