<?php
/**
 * Action handler class
 *
 * @package    buddypress-user-badges
 * @subpackage Handlers
 */

namespace BuddyDev\BuddyPress_User_Badges\Handlers;

// Exit if accessed directly.
use BuddyDev\BuddyPress_User_Badges\Models\Badge;
use BuddyDev\BuddyPress_User_Badges\Models\Badge_Relationship;

// No direct access.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Actions_Handler
 */
class Actions_Handler {

	/**
	 * Class self boot
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup
	 */
	private function setup() {
		add_action( 'show_user_profile', array( $this, 'render_badges' ) );
		add_action( 'edit_user_profile', array( $this, 'render_badges' ) );

		add_action( 'profile_update', array( $this, 'save_user_badges' ) );

		add_action( 'bp_user_query_populate_extras', array( $this, 'populate_user_badges' ), 10, 2 );

		add_action( 'bp_user_badges_badge_deleted', array( $this, 'on_badge_delete' ) );
	}

	/**
	 * Render badges
	 *
	 * @param \WP_User $user user object.
	 */
	public function render_badges( $user ) {

		if ( ! is_super_admin() ) {
			return;
		}

		$badges = Badge::get(
			array(
				'is_active' => 1,
				'source'    => 'custom',
				'per_page'  => 0,
			)
		);

		$user_badges = bp_user_badges_get_user_badges( $user->ID );
		$user_badges = wp_list_pluck( (array) $user_badges, 'id' );

		?>
        <h2><?php esc_html_e( 'Badges', 'buddypress-user-badges' ); ?></h2>
        <table class="form-table">
            <tr class="bp-user-badges-wrap">
				<?php if ( ! empty( $badges ) ) : ?>
                    <th scope="row"><?php esc_html_e( 'Select badges', 'buddypress-user-badges' ); ?></th>
                    <td>
						<?php foreach ( $badges as $badge ) : ?>
                            <label><input type="checkbox" value="<?php echo esc_attr( $badge->id ); ?>"
                                          name="bp_user_badges_user_associated_badge_ids[]" <?php checked( in_array( $badge->id, $user_badges ), true ); ?>>
                                <img class="bp-user-badges-associated-image" width="20"
                                     src="<?php echo esc_url( $badge->image ); ?>"/>
								<?php echo esc_html( $badge->name ); ?>
                            </label>
                            <br>
						<?php endforeach; ?>
                    </td>
				<?php else : ?>
                    <td><?php esc_html_e( 'No badge found', 'buddypress-user-badges' ); ?></td>
				<?php endif; ?>
            </tr>
        </table>
		<?php
	}

	/**
	 * Save user badges
	 *
	 * @param int $user_id User id.
	 */
	public function save_user_badges( $user_id ) {

		if ( ! is_admin() || wp_doing_ajax() || ! function_exists( 'get_current_screen' ) ) {
			return;
		}

		$current_screen = get_current_screen();

        // this must be called from inside wp-admin screen.
		if ( ! isset( $current_screen->id ) || ! in_array( $current_screen->id, array(
				'user-edit',
				'user-edit-network',
				'profile',
			) ) ) {
			return;
		}

		// is the current user super admin?
	    if ( ! is_super_admin() ) {
			return;
		}

		$selected_ids = isset( $_POST['bp_user_badges_user_associated_badge_ids'] ) ? $_POST['bp_user_badges_user_associated_badge_ids'] : array(); // Input var okay.

		if ( empty( $selected_ids ) ) {
			// Remove all previous user badges.
			bp_user_badges_remove_user_badges( $user_id );
		} else {
			$user_badges    = bp_user_badges_get_user_badges( $user_id );
			$user_badge_ids = wp_list_pluck( (array) $user_badges, 'id' );
			$existed_ids    = array_intersect( $selected_ids, $user_badge_ids );
			$diff_ids       = array_diff( $user_badge_ids, $selected_ids );

			foreach ( $selected_ids as $selected_id ) {
				if ( in_array( $selected_id, $existed_ids ) ) {
					continue;
				} else {
					bp_user_badges_add_badge_to_user( $selected_id, $user_id );
				}
			}

			if ( empty( $diff_ids ) ) {
				return;
			}

			Badge_Relationship::destroy(
				array(
					'user_id'  => $user_id,
					'badge_id' => array(
						'op'    => 'IN',
						'value' => $diff_ids,
					),
				)
			);
		}
	}

	/**
	 * Get user's badges and save them to class property
	 *
	 * @param \BP_User_Query $bp_user_query_obj User query object.
	 * @param array          $user_ids List of user ids.
     *
     * @todo we can improve here
	 */
	public function populate_user_badges( $bp_user_query_obj, $user_ids ) {
		// If we are on single user screen no need to populate.
        // CORRECT: WE do need to populate for listings in widget etc.
		//if ( bp_is_user() ) {
		//	return;
		//}

		$bp_user_badges = bp_user_badges();

		$user_ids = explode( ',', $user_ids );

		if ( empty( $user_ids ) ) {
			return;
		}

		$relationships = bp_user_badges_get_relationships(
			array(
				'user_id' => array(
					'op'    => 'IN',
					'value' => $user_ids,
				),
                'per_page'=> 0,
			)
		);

		if ( empty( $relationships ) ) {
			return;
		}

		foreach ( $relationships as $relationship ) {

			$user_badges = empty( $bp_user_badges->user_badges[ $relationship->user_id ] ) ? array() : $bp_user_badges->user_badges[ $relationship->user_id ];

			if ( ! in_array( $relationship->badge_id, $user_badges ) ) {
				$bp_user_badges->user_badges[ $relationship->user_id ][] = $relationship->badge_id;
			}
		}

		foreach ( $user_ids as $user_id ) {
			if ( isset( $bp_user_badges->user_badges[ $user_id ] ) ) {
				continue;
			}
			$bp_user_badges->user_badges[ $user_id ] = array();// make cache hit.
		}
	}

	/**
	 * Remove all relationship with this badge
	 *
	 * @param int $badge_id Badge id.
	 */
	public function on_badge_delete( $badge_id ) {
		bp_user_badges_remove_badge_relationships( $badge_id );
	}
}

